
import numpy as np

def location( bbox ):
    'compute location for the bbox for use with blender python api'
    loc = ( bbox.minpt + bbox.maxpt ) / 2.0
    return [ loc[0], -loc[2], loc[1] ] # flip the coords to cater to blender Z-up coord system

def scale( bbox ):
    'compute scale for the bbox for use with blender python api'
    sc = ( bbox.maxpt - bbox.minpt ) / 2.0
    return [ sc[0], sc[2], sc[1] ] # flip the coords to cater to blender Z-up coord system

def create_viz_script( kdtree ):

    contents = [ 'import bpy' ]

    def process( node ):
        if not node[ 'split_plane' ]:
            return
        bbox = node['left_node'][ 'box' ]
        split_coord, split_dim = node[ 'split_plane' ]
        box_sc = scale( bbox )
        box_loc = location( bbox )
        
        # take the bounding box center, and collapse it along the axis of the split plane
        #if split_dim == 0:
        box_loc[ split_dim ] = split_coord
        contents.append( 'bpy.ops.mesh.primitive_plane_add( location=%s )' % ( box_loc ) )
        contents.append( 'bpy.context.object.scale=%s' % ( box_sc ) )
        if split_dim == 0:
            rot_axis = 'y'
            contents.append( 'bpy.context.object.rotation_euler.%s=%s' % ( rot_axis, np.pi/2.0 ) )
        elif split_dim == 2:
            rot_axis = 'x'
            contents.append( 'bpy.context.object.rotation_euler.%s=%s' % ( rot_axis, np.pi/2.0 ) )

        if node[ 'left_node' ]:
            process( node[ 'left_node' ] )
        if node[ 'right_node' ]:
            process( node[ 'right_node' ] )

    # add the top level bounding box    
    contents.append( 'bpy.ops.mesh.primitive_cube_add( location=%s )' % ( location( kdtree[ 'box' ] ) ) )
    contents.append( 'bpy.context.object.scale=%s' % scale( kdtree[ 'box' ] ) )

    process( kdtree )
    return '\n'.join( contents )

def write_viz_script( kdtree, filename ):
    f = open( filename, 'wt' )
    contents = create_viz_script( kdtree )
    f.write( contents )
    f.close()
