
from numbapro import guvectorize

target = 'gpu'
#target = 'cpu'

@guvectorize( [ 'void( float32[:], float32[:], float32[:], float32[:], float32[:], float32[:] )' ], 
                '(n),(n),(n),(n),(n)->()', target=target )
def ray_tri_intersection( raypos, raydir, v0, v1, v2, outp ):
    e1x, e1y, e1z = v1[0] - v0[0], v1[1] - v0[1], v1[2] - v0[2]
    e2x, e2y, e2z = v2[0] - v0[0], v2[1] - v0[1], v2[2] - v0[2]
    Tx, Ty, Tz    = raypos[0] - v0[0], raypos[1] - v0[1], raypos[2] - v0[2]
    Px = raydir[1] * e2z - raydir[2] * e2y
    Py = raydir[2] * e2x - raydir[0] * e2z
    Pz = raydir[0] * e2y - raydir[1] * e2x
    PdotE1 = Px * e1x + Py * e1y + Pz * e1z
    if PdotE1 < 0.000000001:
        outp[0] = -2.0

    Qx = Ty * e1z - Tz * e1y
    Qy = Tz * e1x - Tx * e1z
    Qz = Tx * e1y - Ty * e1x
    QdotE2 = Qx * e2x + Qy * e2y + Qz * e2z
    PdotT  = Px * Tx + Py * Ty + Pz * Tz
    QdotD  = Qx * raydir[0] + Qy * raydir[1] + Qz * raydir[2]
    bx = QdotE2 / PdotE1
    by = PdotT  / PdotE1
    bz = QdotD  / PdotE1

    if bx > 0.0 and ( by >= 0.0 and by <= 1.0 ) and ( bz >= 0.0 and bz <= 1.0 ):
        outp[0] = bx
    else:
        outp[0] = -1.0
