from   io import BytesIO
from   IPython.display import display, clear_output, Image as IPImage

def tocontainer(func):
    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        return ImageWrapper(result)
    return wrapper

class ImageWrapper(object):
    """ Wraps a PIL Image object and install hook on putpixel to update IPython
        display in realtime. """

    def __init__(self, image):
        self.pil_image = image

    def __getitem__(self, item):
        result = self.pil_image[item]
        if isinstance(result, type(self.pil_image)):
            result = ImageWrapper(result)
        return result

    def putpixel(self, *args, **kwargs):
        ''' install hook on putpixel call so we can dynamically'''
        self.pil_image.putpixel(*args, **kwargs)
        bio = BytesIO()
        self.pil_image.save(bio, format='png')
        clear_output(wait=True)
        display(IPImage(bio.getvalue(), format='png'))       
        
    def __getattr__(self, item):
        result = getattr(self.pil_image, item)
        if callable(result):
            result = tocontainer(result)
        return result

    def __repr__(self):
        return repr(self.pil_image)

class TracerOptions(object):
    
    def __init__( self, scene_file, fast=False, supersample=False, dist=0 ):
        self.scene_file  = scene_file
        self.fast        = fast
        self.supersample = supersample
        self.dist        = dist
    
