
import numpy as np
import operator
import ply

class PlyMesh( object ):

    MAXFLOAT = np.finfo( np.float32 ).max
    MINFLOAT = np.finfo( np.float32 ).min

    def __init__( self, filename ):

        f = open( filename, 'r' ) 
        reader = ply.reader( f )
        fn = operator.attrgetter( 'x', 'y', 'z' )

        self.vertices = np.repeat( np.array( [ [ 0.0, 0.0, 0.0 ] ], dtype=np.float32 ), reader.nrNodes, axis=0 )
        self.faces    = np.repeat( np.array( [ [ 0, 0, 0 ] ], dtype=np.int32 ), reader.nrElements, axis=0 )
        self.minpt = np.array( [ self.MAXFLOAT, self.MAXFLOAT, self.MAXFLOAT ] )
        self.maxpt = np.array( [ self.MINFLOAT, self.MINFLOAT, self.MINFLOAT ] )

        for i, v in enumerate( reader.readNode() ):
            v = np.array( map( np.float, fn( v ) ) )
            for j in xrange( 3 ):
                if self.minpt[j] > v[j]:
                    self.minpt[j] = v[j]
                if self.maxpt[j] < v[j]:
                    self.maxpt[j] = v[j]

            self.vertices[ i ] = v 
 
        faces = list( reader.readElementIndexed() )
        for i, face in enumerate( faces ):
            self.faces[i] = np.array( map( lambda x : int(x) - 1 , face.list ) )
        self.num_vertices = len( self.vertices )
        self.num_faces = len( self.faces )
        
        f.close()

    def __repr__( self ):
        return 'num faces=%s\nnum vertices=%s\nminpt=%s\nmaxpt=%s\n' % ( self.num_faces, self.num_vertices, self.minpt, self.maxpt )


